<?php /** @noinspection PropertyInitializationFlawsInspection */

namespace App\Database;

use App\Database\Utils\ThemeHelperEntity;
use Exception;
use Iterator;
use Jinya\PDOx\Exceptions\InvalidQueryException;
use Jinya\PDOx\Exceptions\NoResultException;

/**
 *
 */
class ThemeAsset extends ThemeHelperEntity
{
    public int $themeId = -1;
    public string $publicPath = '';

    /**
     * Finds a page by name and theme
     *
     * @throws Exceptions\ForeignKeyFailedException
     * @throws InvalidQueryException
     * @throws Exceptions\UniqueFailedException
     * @throws NoResultException
     * @throws NoResultException
     */
    public static function findByThemeAndName(int $themeId, string $name): ?ThemeAsset
    {
        /**
         * @phpstan-ignore-next-line
         */
        return self::fetchByThemeAndName($themeId, $name, 'theme_asset', new self());
    }

    /**
     * Finds the pages for the given theme
     *
     * @param int $themeId
     * @return Iterator<ThemeAsset>
     * @throws Exceptions\ForeignKeyFailedException
     * @throws Exceptions\UniqueFailedException
     * @throws InvalidQueryException
     */
    public static function findByTheme(int $themeId): Iterator
    {
        /**
         * @phpstan-ignore-next-line
         */
        return self::fetchByTheme($themeId, 'theme_asset', new self());
    }

    /**
     * {@inheritDoc}
     * @throws Exception
     */
    public function create(): void
    {
        $this->internalCreate('theme_asset');
    }

    /**
     * {@inheritDoc}
     */
    public function delete(): void
    {
        $this->internalDelete('theme_asset');
    }

    /**
     * {@inheritDoc}
     * @throws Exceptions\UniqueFailedException
     */
    public function update(): void
    {
        $this->internalUpdate('theme_asset');
    }

    /**
     * @return array<string>
     */
    public function format(): array
    {
        return [];
    }
}
