<?php

namespace App\Web\Actions\Authentication;

use App\Database\Artist;
use App\Database\Exceptions\ForeignKeyFailedException;
use App\Database\Exceptions\UniqueFailedException;
use App\Web\Actions\Action;
use App\Web\Exceptions\MissingFieldsException;
use App\Web\Middleware\AuthorizationMiddleware;
use Jinya\PDOx\Exceptions\InvalidQueryException;
use Psr\Http\Message\ResponseInterface as Response;
use Slim\Exception\HttpForbiddenException;

/**
 *
 */
class ChangePasswordAction extends Action
{
    /**
     * @return Response
     * @throws ForeignKeyFailedException
     * @throws HttpForbiddenException
     * @throws InvalidQueryException
     * @throws MissingFieldsException
     * @throws UniqueFailedException
     */
    protected function action(): Response
    {
        $password = $this->body['password'];
        $oldPassword = $this->body['oldPassword'];
        /** @var Artist $currentArtist */
        $currentArtist = $this->request->getAttribute(AuthorizationMiddleware::LOGGED_IN_ARTIST);

        if ($password) {
            if (!$currentArtist->changePassword($oldPassword, $password)) {
                throw new HttpForbiddenException($this->request, 'Old password is invalid');
            }

            return $this->noContent();
        }

        throw new MissingFieldsException($this->request, ['password']);
    }
}
